@@ -190,36 +190,41 @@ Base.size(ei::EachIndex) = ei.dims
 Base.getindex(ei::EachIndex, i::Int) = ei.indices[i]
 Base.IndexStyle(::Type{<:EachIndex}) = Base.IndexLinear()
 
-function Base.findfirst(f::Function, A::AnyGPUArray)
-    isempty(A) && return nothing
-    indices = EachIndex(A)
-    dummy_index = first(indices)
-
-    # given two pairs of (istrue, index), return the one with the smallest index
-    function reduction(t1, t2)
+function findfirstlast_reduction(op_and_dummy, t1, t2)
+    op, dummy_index = op_and_dummy
+    (x, i), (y, j) = t1, t2
+    if op(i, j)
+        t1, t2 = t2, t1
         (x, i), (y, j) = t1, t2
-        if i > j
-            t1, t2 = t2, t1
-            (x, i), (y, j) = t1, t2
-        end
-        x && return t1
-        y && return t2
-        return (false, dummy_index)
-    end
-
-    res = mapreduce((x, y)->(f(x), y), reduction, A, indices;
-                    init = (false, dummy_index))
-    if res[1]
-        # out of consistency with Base.findarray, return a CartesianIndex
-        # when the input is a multidimensional array
-        ndims(A) == 1 && return res[2]
-        return CartesianIndices(A)[res[2]]
-    else
-        return nothing
     end
+    x && return t1
+    y && return t2
+    return (false, dummy_index)
 end
 
+for (find_f, op, dummy_f) in ((:(Base.findfirst), :>, :first), (:(Base.findlast), :<, :last))
+    @eval begin
+        function $find_f(f::Function, A::AnyGPUArray)
+            isempty(A) && return nothing
+            indices = EachIndex(A)
+            dummy_index = $dummy_f(indices)
+
+            # given two pairs of (istrue, index), return the one with the smallest index
+            res = mapreduce((x, y)->(f(x), y), (a, b)->findfirstlast_reduction(($op, dummy_index), a, b), A, indices;
+                            init = (false, dummy_index))
+            if res[1]
+                # out of consistency with Base.findarray, return a CartesianIndex
+                # when the input is a multidimensional array
+                ndims(A) == 1 && return res[2]
+                return CartesianIndices(A)[res[2]]
+            else
+                return nothing
+            end
+        end
+    end
+end
 Base.findfirst(A::AnyGPUArray{Bool}) = findfirst(identity, A)
+Base.findlast(A::AnyGPUArray{Bool})  = findlast(identity, A)
 
 function findminmax(binop, A::AnyGPUArray; init, dims)
     indices = EachIndex(A)
